#include "MySiTCP_CNTPIX.h"

void MySiTCP_CNTPIX::DAQStart(bool start){
  RBCPskeleton(0xff, 0x80, 0x0, 0x1, 0x0);

  sndData[0] = start ? 1 : 0;
  memcpy(sndBuf+sizeof(sndHeader), sndData, 1);

  int len = 0;
  len = sendto(udpsock, sndBuf, 1 + sizeof(sndHeader), 0,
 (struct sockaddr *)&udpAddr, sizeof(udpAddr));
  if(len < 0){
    perror("Send DAQ Start Signal");
    exit(1);
  }
  if(start)std::cout<<"Send DAQ Start Signal to SiTCP Board... Done"<<std::endl;
  else std::cout<<"Send DAQ Stop Signal to SiTCP Board... Done"<<std::endl;
  rcvRBCP_ACK(0);
}

void MySiTCP_CNTPIX::SetCSRIniVal(int window, int TEST_IN, int dach, int dacl){
  if(window<0 || 1<window){
    std::cout<<"window mode flag should be 0-1"<<std::endl;
    exit(1);
  }
  if(TEST_IN<0 || 1<TEST_IN){
    std::cout<<"TEST_IN flag should be 0-1"<<std::endl;
    exit(1);
  }
  if(dach<0 || 8<dach){
    std::cout<<"DAC_H fine tune bit should be 0-8"<<std::endl;
    exit(1);
  }
  if(dacl<0 || 8<dacl){
    std::cout<<"DAC_L fine tune bit should be 0-8"<<std::endl;
    exit(1);
  }
  
  RBCPskeleton(0xff, 0x80, 0x7, 0x1, 0x7);
  sndData[0] = window ? 0x80 : 0x00;
  sndData[0] = TEST_IN ? sndData[0]+0x08 : sndData[0]+0x00;
  sndData[0] = sndData[0]+((0x07&dach)<<4);
  sndData[0] = sndData[0]+(0x07&dacl);
  printf("CSR ini: sndData[0]=%x\n",sndData[0]);
  memcpy(sndBuf+sizeof(sndHeader), sndData, 1);
  int len = 0;
  len = sendto(udpsock, sndBuf, 1 + sizeof(sndHeader), 0, (struct sockaddr *)&udpAddr, sizeof(udpAddr));
  if(len < 0){
    perror("Set csr initial value");
    exit(1);
  }
  rcvRBCP_ACK(0);
  usleep(1000000);
  std::cout << " Set csr_ini_val" << std::endl;
}

void MySiTCP_CNTPIX::SetCSRIniValpix(int window, int TEST_IN, int dach, int dacl,int chno){

  int len=0;
  if(window<0 || 1<window){
    std::cout<<"window mode flag should be 0-1"<<std::endl;
    exit(1);
  }
  if(TEST_IN<0 || 1<TEST_IN){
    std::cout<<"TEST_IN flag should be 0-1"<<std::endl;
    exit(1);
  }
  if(dach<0 || 8<dach){
    std::cout<<"DAC_H fine tune bit should be 0-8"<<std::endl;
    exit(1);
  }
  if(dacl<0 || 8<dacl){
    std::cout<<"DAC_L fine tune bit should be 0-8"<<std::endl;
    exit(1);
  }
  if(chno<0 || 16384<chno){
    std::cout<<"chno should be 0-16384"<<std::endl;
    exit(1);
  }

  //send addr10 signal
  RBCPskeleton(0xff, 0x80, 0xa, 0x1, 0xa);
  sndData[0] = int(chno/128);
  memcpy(sndBuf+sizeof(sndHeader), sndData, 1);
  len = 0;
  len = sendto(udpsock, sndBuf, 1 + sizeof(sndHeader), 0,
 	       (struct sockaddr *)&udpAddr, sizeof(udpAddr));
  if(len < 0){
    perror("Send DAQ Interbal (ADDR[10])");
    exit(1);
  }
  rcvRBCP_ACK(0);
  usleep(0.1);
  
  //send addr11 signal
  RBCPskeleton(0xff, 0x80, 0xb, 0x1, 0xb);
  sndData[0] = chno%128;
  memcpy(sndBuf+sizeof(sndHeader), sndData, 1);
  len = 0;
  len = sendto(udpsock, sndBuf, 1 + sizeof(sndHeader), 0,
 	       (struct sockaddr *)&udpAddr, sizeof(udpAddr));
  if(len < 0){
    perror("Send DAQ Interbal (ADDR[11])");
    exit(1);
  }
  rcvRBCP_ACK(0);
  usleep(0.1);
  
  RBCPskeleton(0xff, 0x80, 0x7, 0x1, 0x7);
  sndData[0] = window ? 0x80 : 0x00;
  sndData[0] = TEST_IN ? sndData[0]+0x08 : sndData[0]+0x00;
  sndData[0] = sndData[0]+((0x07&dach)<<4);
  sndData[0] = sndData[0]+(0x07&dacl);
  //  printf("csrinipix : sndData[0]=%x\n",sndData[0]);
  memcpy(sndBuf+sizeof(sndHeader), sndData, 1);
  len = 0;
  len = sendto(udpsock, sndBuf, 1 + sizeof(sndHeader), 0, (struct sockaddr *)&udpAddr, sizeof(udpAddr));
  if(len < 0){
    perror("Set csr initial value");
    exit(1);
  }
  rcvRBCP_ACK(0);
  usleep(0.1);  
  
  if(chno%1024==0){
    //    rcvRBCP_ACK(0);
    //    usleep(50);  
    std::cout << "Set csr_ini_valpix " << chno/128 << " " << chno%128 << " " << sndData[0] << std::endl;
  }
}


void MySiTCP_CNTPIX::SetCSRchno(int chno){
  int len=0;  
  if(chno<0 || 16384<chno){
    std::cout<<"chno should be 0-16384"<<std::endl;
    exit(1);
  }
  //send addr10 signal
  RBCPskeleton(0xff, 0x80, 0xa, 0x1, 0xa);
  sndData[0] = int(chno/128);
  memcpy(sndBuf+sizeof(sndHeader), sndData, 1);
  len = 0;
  len = sendto(udpsock, sndBuf, 1 + sizeof(sndHeader), 0,
 	       (struct sockaddr *)&udpAddr, sizeof(udpAddr));
  if(len < 0){
    perror("Send DAQ Interbal (ADDR[10])");
    exit(1);
  }
  //  rcvRBCP_ACK(0);
  //  usleep(50);
  //send addr11 signal
  RBCPskeleton(0xff, 0x80, 0xb, 0x1, 0xb);
  sndData[0] = chno%128;
  memcpy(sndBuf+sizeof(sndHeader), sndData, 1);
  len = 0;
  len = sendto(udpsock, sndBuf, 1 + sizeof(sndHeader), 0,
 	       (struct sockaddr *)&udpAddr, sizeof(udpAddr));
  if(len < 0){
    perror("Send DAQ Interbal (ADDR[11])");
    exit(1);
  }
  //rcvRBCP_ACK(0);
  //usleep(5000);
  //std::cout << " Set ch " << chno/128 << " " << chno%128 << std::endl;
}

void MySiTCP_CNTPIX::SetDAQInterbal(int time){
  if(!((10<=time && time<=16777216) || time==0)){
    std::cout<<"DAQ Interbal value should be 10-16777216*40ns"<<std::endl;
    std::cout<<time<<" isn't valid value"<<std::endl;
    exit(1);
  }
  char addr1, addr2, addr3;
  addr1 = (char){(unsigned int)time >> 16};
  addr2 = (char){(unsigned int)time >> 8};
  addr3 = (char){(unsigned int)time & 0x000000FF};
  // std::cout<<"Set DAQ_interval:"<<std::endl;
  //  std::cout<<(unsigned short)addr1<<std::endl;
  //  std::cout<<(unsigned short)addr2<<std::endl;
  //  std::cout<<(unsigned short)addr3<<std::endl;
  usleep(50);

  //send addr1 signal
  RBCPskeleton(0xff, 0x80, 0x1, 0x1, 0x1);
  sndData[0] = time ? (unsigned short)addr1 : 0;
  memcpy(sndBuf+sizeof(sndHeader), sndData, 1);
  int len = 0;
  len = sendto(udpsock, sndBuf, 1 + sizeof(sndHeader), 0,
	       (struct sockaddr *)&udpAddr, sizeof(udpAddr));
  if(len < 0){
    perror("Send DAQ Interbal (ADDR[1])");
    exit(1);
  }
  rcvRBCP_ACK(0);
  usleep(50);

  //send addr2 signal
   RBCPskeleton(0xff, 0x80, 0x2, 0x1, 0x2);
   sndData[0] = time ? (unsigned short)addr2 : 0;
   memcpy(sndBuf+sizeof(sndHeader), sndData, 1);
   len = 0;
   len = sendto(udpsock, sndBuf, 1 + sizeof(sndHeader), 0,
 	       (struct sockaddr *)&udpAddr, sizeof(udpAddr));
   if(len < 0){
     perror("Send DAQ Interbal (ADDR[2])");
     exit(1);
   }
   rcvRBCP_ACK(0);
   usleep(50);

  //send addr3 signal
   RBCPskeleton(0xff, 0x80, 0x3, 0x1, 0x3);
   sndData[0] = time ? (unsigned short)addr3 : 0;
   memcpy(sndBuf+sizeof(sndHeader), sndData, 1);
   len = 0;
   len = sendto(udpsock, sndBuf, 1 + sizeof(sndHeader), 0,
 	       (struct sockaddr *)&udpAddr, sizeof(udpAddr));
   if(len < 0){
     perror("Send DAQ Interbal (ADDR[3])");
     exit(1);
   }
   rcvRBCP_ACK(0);
   usleep(50);
    
   std::cout << "Set daq_interval= " << time<< std::endl;

} 

void MySiTCP_CNTPIX::SetRAENHiTime(int time){
  if(!(0<=time && time<=16777216)){
    std::cout<<"RAEN Hi Time value should be 0-16777216*40ns"<<std::endl;
    std::cout<<time<<" isn't valid value"<<std::endl;
    exit(1);
  }
  char addr4, addr5, addr6;
  addr4 = (char){(unsigned int)time >> 16};
  addr5 = (char){(unsigned int)time >> 8};
  addr6 = (char){(unsigned int)time & 0x000000FF};
  // std::cout<<"Set RAENHiTime:"<<std::endl;
  //  std::cout<<(unsigned short)addr4<<std::endl;
  //  std::cout<<(unsigned short)addr5<<std::endl;
  //  std::cout<<(unsigned short)addr6<<std::endl;
  usleep(50);

  //send addr4 signal
  RBCPskeleton(0xff, 0x80, 0x4, 0x1, 0x4);
  sndData[0] = time ? (unsigned short)addr4 : 0;
  memcpy(sndBuf+sizeof(sndHeader), sndData, 1);
  int len = 0;
  len = sendto(udpsock, sndBuf, 1 + sizeof(sndHeader), 0,
	       (struct sockaddr *)&udpAddr, sizeof(udpAddr));
  if(len < 0){
    perror("Send RAEN Hi Time (ADDR[4])");
    exit(1);
  }
  rcvRBCP_ACK(0);
  usleep(50);

  //send addr5 signal
   RBCPskeleton(0xff, 0x80, 0x5, 0x1, 0x5);
   sndData[0] = time ? (unsigned short)addr5 : 0;
   memcpy(sndBuf+sizeof(sndHeader), sndData, 1);
   len = 0;
   len = sendto(udpsock, sndBuf, 1 + sizeof(sndHeader), 0,
 	       (struct sockaddr *)&udpAddr, sizeof(udpAddr));
   if(len < 0){
     perror("Send RAEN Hi Time (ADDR[5])");
     exit(1);
   }
   rcvRBCP_ACK(0);
   usleep(50);
   //std::cout << ":end of addr5 sending:" << std::endl;

  //send addr6 signal
   RBCPskeleton(0xff, 0x80, 0x6, 0x1, 0x6);
   sndData[0] = time ? (unsigned short)addr6 : 0;
   memcpy(sndBuf+sizeof(sndHeader), sndData, 1);
   len = 0;
   len = sendto(udpsock, sndBuf, 1 + sizeof(sndHeader), 0,
 	       (struct sockaddr *)&udpAddr, sizeof(udpAddr));
   if(len < 0){
     perror("Send RAEN Hi Time (ADDR[6])");
     exit(1);
   }
   rcvRBCP_ACK(0);
   usleep(50);

   std::cout << "Set RAEN_hi_time= " << time <<std::endl;    

} 

int MySiTCP_CNTPIX::DAC_cnt(int ch, int mV){
  if(!(0<=mV && mV<=3000)){
    std::cout<<"DAC output should be 0-3000[mV]"<<std::endl;
    std::cout<<mV<<"[mV] isn't valid value"<<std::endl;
    std::cout<<"Now, No output!"<<std::endl;
    return -1;
  }
  if(!(0<=ch && ch<=4)){
    std::cout<<"DAC ch is 0-3"<<std::endl;
    std::cout<<ch<<" isn't valid value"<<std::endl;
  }
  char addr8, addr9;
  double bit;
  bit = (double)mV*4095/3300*1.1;//1.1 is unknown factor to calibrate            
  addr8 = (char){(unsigned short)floor(bit) >> 8};
  addr9 = (char){(unsigned short)floor(bit) & 0x00FF};
  if(ch==4){
    addr8 = addr8 + (3<<4);
  }else{
    addr8 = addr8 + (1<<4);
  }
  addr8 = addr8 + (ch<<6);
  // std::cout<<(unsigned short)addr8<<std::endl;
  //std::cout<<(unsigned short)addr9<<std::endl;                                 
  usleep(50);

  //send addr8 signal                                                            
  RBCPskeleton(0xff, 0x80, 0x8, 0x1, 0x8);
  sndData[0] = (unsigned short)addr8;
  memcpy(sndBuf+sizeof(sndHeader), sndData, 1);
  int len = 0;
  len = sendto(udpsock, sndBuf, 1 + sizeof(sndHeader), 0,
               (struct sockaddr *)&udpAddr, sizeof(udpAddr));
  if(len < 0){
    perror("Send DAC Control Signal (ADDR[8])");
    exit(1);
  }
  rcvRBCP_ACK(0);
  usleep(50);

  //send addr9 signal                                                            
   RBCPskeleton(0xff, 0x80, 0x9, 0x1, 0x9);
   sndData[0] = (unsigned short)addr9;
   memcpy(sndBuf+sizeof(sndHeader), sndData, 1);
   len = 0;
   len = sendto(udpsock, sndBuf, 1 + sizeof(sndHeader), 0,
               (struct sockaddr *)&udpAddr, sizeof(udpAddr));
   if(len < 0){
     perror("Send DAC Control Signal (ADDR[9])");
     exit(1);
   }
   rcvRBCP_ACK(0);
   usleep(1000000);

   std::cout << "Set DAC= " << ch <<" " << mV << std::endl;

  return 0;
}

int MySiTCP_CNTPIX::TakeDatAt(unsigned short *data){
  bool ret = true;
  int rlen = 0;
  rlen = recv(tcpsock, &tmp[0][0], sizeof(tmp), MSG_WAITALL);
  //rlen = recv(tcpsock, &tmp[0][0], sizeof(tmp), 0);    
  //std::cout<<"receive length = "<<rlen<<std::endl;     
  if(rlen < 0){
    perror("receive rlen<0");
    ret = false;
    //    exit(1);  
  }else if(rlen != 32768){
    std::cout<<"receive failuer : rlen = "<<rlen<<std::endl;
  }
  memcpy(data, tmp, sizeof(tmp));
  return rlen;
}

int MySiTCP_CNTPIX::tcpselect(){
  //return 0 : There is a keyboard input.    
  //return 1 : tcp receiving data ready
  //return -1 : ERROR

  memcpy(&fds, &readfds, sizeof(fd_set));
  timeout.tv_sec = 3;
  timeout.tv_usec = 0;
  //  int n = select(tcpsock+1, &fds, NULL, NULL, &timeout);
  select(tcpsock+1, &fds, NULL, NULL, NULL);
  if(FD_ISSET(tcpsock, &fds)){
    return 1;
  }else if(FD_ISSET(0, &fds)){
    return 0;
  }else{
    return -1;
  }
}


